;;; arabic.el --- pre-loaded support for Arabic.

;; Copyright (C) 1992,93,94,95 Free Software Foundation, Inc.
;; Copyright (C) 1995 Amdahl Corporation.
;; Copyright (C) 1995 Sun Microsystems.

;; This file is part of XEmacs.

;; XEmacs is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; XEmacs is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with XEmacs; see the file COPYING.  If not, write to the 
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Synched up with: Mule 2.3.

;;; Code:

;; Three character sets for Arabic
(make-charset 'arabic-digit "Arabic digits"
	      '(registry "MuleArabic-0"
		dimension 1
		chars 94
		final ?2
		graphic 0
		direction l2r
		))

(make-charset 'arabic-1-column "Arabic 1-column"
	      '(registry "MuleArabic-1"
		dimension 1
		chars 94
		final ?3
		graphic 0
		direction r2l
		))

(make-charset 'arabic-2-column "Arabic 2-column"
	      '(registry "MuleArabic-2"
		dimension 1
		chars 94
		final ?4
		graphic 0
		direction r2l
		))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ARABIC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; (define-language-environment 'arabic
;;   "Arabic"
;;   (lambda ()
;;     (require 'arabic)))

;;; arabic.el ends here
